package jumpstart.business.domain.examples;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Version;

import org.joda.time.DateMidnight;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.LocalDate;
import org.joda.time.LocalDateTime;
import org.joda.time.LocalTime;

/**
 * The DateStuff entity.
 */
@Entity
@SuppressWarnings("serial")
public class DateStuff {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@Column(nullable = false)
	protected Long id;

	@Version
	@Column(nullable = false)
	protected Integer version;

	// Traditional java-style Date fields

	@Temporal(TemporalType.TIMESTAMP)
	protected Date aTimestamp;

	@Temporal(TemporalType.DATE)
	protected Date aDate;

	@Temporal(TemporalType.TIME)
	protected Date aTime;

	// Joda-time fields

	protected java.sql.Timestamp aDateTime;

	protected java.sql.Timestamp aDateTimeWithTZ;

	protected String aDateTimeTZ;

	protected java.sql.Date aDateMidnight;

	protected java.sql.Date aDateMidnightWithTZ;

	protected String aDateMidnightTZ;

	protected java.sql.Timestamp aLocalDateTime;

	protected java.sql.Date aLocalDate;

	protected java.sql.Time aLocalTimeAsTime;

	protected Integer aLocalTimeAsMillis;

	protected String aLocalTimeAsString;

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("DateStuff: [");
		buf.append("id=" + id + ", ");
		buf.append("aTimestamp=" + aTimestamp + ", ");
		buf.append("aDate=" + aDate + ", ");
		buf.append("aTime=" + aTime + ", ");
		buf.append("aDateTime=" + aDateTime + ", ");
		buf.append("aDateTimeWithTZ=" + aDateTimeWithTZ + ", ");
		buf.append("aDateTimeTZ=" + aDateTimeTZ + ", ");
		buf.append("aDateMidnight=" + aDateMidnight + ", ");
		buf.append("aLocalDateTime=" + aLocalDateTime + ", ");
		buf.append("aLocalDate=" + aLocalDate + ", ");
		buf.append("aLocalTimeAsTime=" + aLocalTimeAsTime + ", ");
		buf.append("aLocalTimeAsMillis=" + aLocalTimeAsMillis + ", ");
		buf.append("aLocalTimeAsString=" + aLocalTimeAsString + ", ");
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html

	@Override
	public boolean equals(Object obj) {
		return (obj == this) || (obj instanceof DateStuff) && id != null && id.equals(((DateStuff) obj).id);
	}

	// The need for a hashCode() method is discussed at http://www.hibernate.org/109.html

	@Override
	public int hashCode() {
		return getId() == null ? super.hashCode() : getId().hashCode();
	}

	public Long getId() {
		return id;
	}

	public Integer getVersion() {
		return version;
	}

	public void setVersion(Integer version) {
		this.version = version;
	}

	public Date getADate() {
		return aDate;
	}

	public void setADate(Date aDate) {
		this.aDate = aDate;
	}

	public Date getATime() {
		return aTime;
	}

	public void setATime(Date aTime) {
		this.aTime = aTime;
	}

	public Date getATimestamp() {
		return aTimestamp;
	}

	public void setATimestamp(Date aTimestamp) {
		this.aTimestamp = aTimestamp;
	}

	public DateTime getADateTime() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aDateTime == null ? null : new DateTime(aDateTime));
	}

	public void setADateTime(DateTime dt) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aDateTime = (dt == null ? null : new java.sql.Timestamp(dt.getMillis()));
	}

	public DateTime getADateTimeWithTZ() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aDateTimeWithTZ == null ? null : new DateTime(aDateTimeWithTZ, DateTimeZone.forID(aDateTimeTZ)));
	}

	public void setADateTimeWithTZ(DateTime dt) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aDateTimeWithTZ = (dt == null ? null : new java.sql.Timestamp(dt.getMillis()));
		this.aDateTimeTZ = (dt == null ? null : dt.getZone().getID());
	}

	public DateMidnight getADateMidnight() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aDateMidnight == null ? null : new DateMidnight(aDateMidnight));
	}

	public void setADateMidnight(DateMidnight dm) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aDateMidnight = (dm == null ? null : new java.sql.Date(dm.getMillis()));
	}

	public DateMidnight getADateMidnightWithTZ() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aDateMidnightWithTZ == null ? null : new DateMidnight(aDateMidnightWithTZ, DateTimeZone
				.forID(aDateMidnightTZ)));
	}

	public void setADateMidnightWithTZ(DateMidnight dm) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aDateMidnightWithTZ = (dm == null ? null : new java.sql.Date(dm.getMillis()));
		this.aDateMidnightTZ = (dm == null ? null : dm.getZone().getID());
	}

	public LocalDateTime getALocalDateTime() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aLocalDateTime == null ? null : LocalDateTime.fromDateFields(aLocalDateTime));
	}

	public void setALocalDateTime(LocalDateTime ldt) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aLocalDateTime = (ldt == null ? null : new java.sql.Timestamp(ldt.toDateTime().getMillis()));
	}

	public LocalDate getALocalDate() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aLocalDate == null ? null : LocalDate.fromDateFields(aLocalDate));
	}

	public void setALocalDate(LocalDate ld) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aLocalDate = (ld == null ? null : new java.sql.Date(ld.toDateTimeAtStartOfDay().getMillis()));
	}

	public LocalTime getALocalTimeAsTime() {
		// TODO - confirm this conversion always works, esp. across timezones
		return (aLocalTimeAsTime == null ? null : new LocalTime(aLocalTimeAsTime, DateTimeZone.UTC));
	}

	public void setALocalTimeAsTime(LocalTime lt) {
		// TODO - confirm this conversion always works, esp. across timezones
		this.aLocalTimeAsTime = (lt == null ? null : new java.sql.Time(lt.getMillisOfDay()));
	}

	public LocalTime getALocalTimeAsMillis() {
		return (aLocalTimeAsMillis == null ? null : LocalTime.fromMillisOfDay(aLocalTimeAsMillis));
	}

	public void setALocalTimeAsMillis(LocalTime lt) {
		this.aLocalTimeAsMillis = (lt == null ? null : new Integer(lt.getMillisOfDay()));
	}

	public LocalTime getALocalTimeAsString() {
		return (aLocalTimeAsString == null ? null : new LocalTime(aLocalTimeAsString));
	}

	public void setALocalTimeAsString(LocalTime lt) {
		this.aLocalTimeAsString = (lt == null ? null : lt.toString());
	}

}
