package jumpstart.business.domain.examples;

import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.business.domain.examples.iface.IPersonServiceRemote;

@Stateless
@Local(IPersonServiceLocal.class)
@Remote(IPersonServiceRemote.class)
public class PersonService implements IPersonServiceLocal, IPersonServiceRemote {

	@PersistenceContext(unitName = "jumpstart")
	protected EntityManager _em;

	public Person findPerson(Long id) {
		return _em.find(Person.class, id);
	}

	@SuppressWarnings("unchecked")
	public List<Person> findPersons() {
		return _em.createQuery("select p from Person p order by p.id").getResultList();
	}

	public Person createPerson(Person person) {
		_em.persist(person);
		return person;
	}

	public void createPersons(List<Person> persons) {
		for (Person person : persons) {
			_em.persist(person);
		}
	}

	public void changePerson(Person person) {
		_em.merge(person);
	}

	public void changePersons(List<Person> persons) {
		for (Person person : persons) {
			_em.merge(person);
		}
	}

	public void bulkEditPersons(List<Person> personsToCreate, List<Person> personsToChange, List<Person> personsToDelete) {
		for (Person person : personsToCreate) {
			_em.persist(person);
		}
		for (Person person : personsToChange) {
			_em.merge(person);
		}
		for (Person person : personsToDelete) {
			person = _em.merge(person);
			_em.remove(person);
		}
	}
}
