package jumpstart.business.domain.workout;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Version;

import jumpstart.business.commons.exception.BusinessException;
import jumpstart.business.commons.exception.GenericBusinessException;
import jumpstart.business.commons.exception.ValueRequiredException;
import jumpstart.business.domain.base.BaseEntity;
import jumpstart.util.StringUtil;

/**
 * Teacher is an AGGREGRATION relationship with Department. Department is the parent. Teacher has an ASSOCIATION
 * relationship with Room. Teacher OPTIONALLY has a "favourite" Room.
 */
@Entity
@SuppressWarnings("serial")
public class Teacher extends BaseEntity {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	protected Long id;

	@Version
	@Column(nullable = false)
	protected Integer version;

	@Column(length = 20, nullable = false)
	protected String name;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "departmentId", nullable = true)
	protected Department department;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "favouriteRoomId", nullable = true)
	protected Room favouriteRoom;

	public Teacher() {
	}

	public Teacher(String name, Department department, Room favouriteRoom) {
		super();
		this.name = name;
		this.department = department;
		if (department != null) {
			department.addTeacher(this);
		}
		this.favouriteRoom = favouriteRoom;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("Teacher: [");
		buf.append("id=" + id + ", ");
		buf.append("name=" + name + ", ");
		buf.append("department=" + toStringLazy(department, "getId") + ", ");
		buf.append("favouriteRoom=" + toStringLazy(favouriteRoom, "getId") + ", ");
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html

	@Override
	public boolean equals(Object obj) {
		return (obj == this) || (obj instanceof Teacher) && id != null && id.equals(((Teacher) obj).id);
	}

	// The need for an equals() method is discussed at http://www.hibernate.org/109.html

	@Override
	public int hashCode() {
		return getId() == null ? super.hashCode() : getId().hashCode();
	}

	@Override
	public Serializable getIdForMessages() {
		return getId();
	}

	@PrePersist
	@PreUpdate
	public void validate() throws BusinessException {

		if (StringUtil.isEmpty(name)) {
			throw new ValueRequiredException(this, "Teacher_name");
		}

		// This is a silly validation, put in here solely to help GenericBusinessExceptionTest!

		if (name.equals("silly")) {
			throw new GenericBusinessException("Teacher_name_is_silly");
		}

	}

	public Long getId() {
		return id;
	}

	public Integer getVersion() {
		return version;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Department getDepartment() {
		return department;
	}

	public void setDepartment(Department department) {
		if (this.department != department) {
			if (this.department != null) {
				/* Maintain the bidirectional relationship with my parent, Department. */
				this.department.removeTeacher(this);
			}
			this.department = department;
			if (department != null) {
				/* Maintain the bidirectional relationship with my parent, Department. */
				department.addTeacher(this);
			}
		}
	}

	public Room getFavouriteRoom() {
		return favouriteRoom;
	}

	public void setFavouriteRoom(Room favouriteRoom) {
		this.favouriteRoom = favouriteRoom;
	}
}
