package jumpstart.business;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.NoInitialContextException;

import jumpstart.business.commons.exception.BusinessException;
import jumpstart.business.commons.interpreter.BusinessServiceExceptionInterpreter;
import jumpstart.business.domain.security.iface.ISecurityFinderSvcRemote;
import jumpstart.business.domain.security.iface.ISecurityManagerSvcRemote;
import jumpstart.business.domain.workout.iface.IWorkoutServiceRemote;
import jumpstart.util.EJBProviderEnum;
import jumpstart.util.EJBProviderUtil;

import org.junit.Before;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BaseTest {

	static protected boolean _servicesLocated = false;

	static protected ISecurityFinderSvcRemote _securityFinderSvc;
	static protected ISecurityManagerSvcRemote _securityManagerSvc;
	static protected IWorkoutServiceRemote _workoutService;

	static protected final Logger _logger = LoggerFactory.getLogger(BusinessServiceExceptionInterpreter.class);
	static protected BusinessServiceExceptionInterpreter _interpreter = new BusinessServiceExceptionInterpreter();

	static public void locateServices() {
		EJBProviderEnum ejbProvider = EJBProviderUtil.detectEJBProvider(_logger);
		InitialContext initialContext = null;

		try {
			initialContext = new InitialContext();
			_logger.info("InitialContext environment = " + initialContext.getEnvironment());

			if (ejbProvider == EJBProviderEnum.OPENEJB_LOCAL) {
				_securityFinderSvc = (ISecurityFinderSvcRemote) initialContext.lookup("SecurityFinderSvcLocal");
				_securityManagerSvc = (ISecurityManagerSvcRemote) initialContext.lookup("SecurityManagerSvcLocal");
				_workoutService = (IWorkoutServiceRemote) initialContext.lookup("WorkoutServiceLocal");
			}
			else if (ejbProvider == EJBProviderEnum.OPENEJB_REMOTE) {
				_securityFinderSvc = (ISecurityFinderSvcRemote) initialContext.lookup("SecurityFinderSvcRemote");
				_securityManagerSvc = (ISecurityManagerSvcRemote) initialContext.lookup("SecurityManagerSvcRemote");
				_workoutService = (IWorkoutServiceRemote) initialContext.lookup("WorkoutServiceRemote");
			}
			else if (ejbProvider == EJBProviderEnum.JBOSS_REMOTE) {
				_securityFinderSvc = (ISecurityFinderSvcRemote) initialContext
						.lookup("jumpstart/SecurityFinderSvc/remote");
				_securityManagerSvc = (ISecurityManagerSvcRemote) initialContext
						.lookup("jumpstart/SecurityManagerSvc/remote");
				_workoutService = (IWorkoutServiceRemote) initialContext.lookup("jumpstart/WorkoutService/remote");
			}
			else if (ejbProvider == EJBProviderEnum.GLASSFISH_REMOTE) {
				_securityFinderSvc = (ISecurityFinderSvcRemote) initialContext
						.lookup("jumpstart.business.domain.security.iface.ISecurityFinderSvcRemote");
				_securityManagerSvc = (ISecurityManagerSvcRemote) initialContext
						.lookup("jumpstart.business.domain.security.iface.ISecurityManagerSvcRemote");
				_workoutService = (IWorkoutServiceRemote) initialContext
						.lookup("jumpstart.business.domain.workout.iface.IWorkoutServiceRemote");
			}
			else {
				throw new IllegalStateException("Don't know how to use ejbProvider = " + ejbProvider);
			}
		}
		catch (NoInitialContextException e) {
			throw new RuntimeException(
					"Could not locate services.  Is jndi.properties missing from the testing classpath?", e);
		}
		catch (NamingException e) {
			throw new RuntimeException(
					"Could not locate services.  Is ejb server started?  If using jboss, is jbossall-client.jar missing from the testing classpath?",
					e);
		}

	}

	@Before
	public void setUp() throws Exception {

		if (!_servicesLocated) {
			locateServices();
			_servicesLocated = true;
		}

		deleteAllWorkoutEntities();
	}

	public void deleteAllWorkoutEntities() throws BusinessException {
		_workoutService.deleteAllStudents();
		_workoutService.deleteAllTeachers();
		_workoutService.deleteAllDepartments();
		_workoutService.removeAllRooms();
		_workoutService.deleteAllBuildings();
		_workoutService.deleteAllStringThings();
	}

}
