package jumpstart.web.services;

import org.apache.tapestry5.ioc.MappedConfiguration;
import org.apache.tapestry5.ioc.OrderedConfiguration;
import org.apache.tapestry5.ioc.ServiceBinder;
import org.apache.tapestry5.ioc.annotations.InjectService;
import org.apache.tapestry5.services.BindingFactory;
import org.apache.tapestry5.services.BindingSource;
import org.apache.tapestry5.services.Dispatcher;

public class AppModule {

	// Add 2 services to those provided by Tapestry.
	// - IBusinessServicesLocator which is used by pages which ask Tapestry to @Inject it.
	// - ProtectedPageGateKeeper is contributed, below, to Tapestry's MasterDispatcher.

	public static void bind(ServiceBinder binder) {
		binder.bind(IBusinessServicesLocator.class, BusinessServicesLocator.class);
		binder.bind(ProtectedPageGateKeeper.class).withId("ProtectedPageGateKeeper");
	}

	// Tell Tapestry which locales we support.
	// We do this by contributing to Tapestry's ApplicationDefaults service.

	public static void contributeApplicationDefaults(MappedConfiguration<String, String> configuration) {
		configuration.add("tapestry.supported-locales", "en_US,en_GB,fr");
	}

	// Tell Tapestry how to handle a new binding prefix, "list:" (we use "list:" in at least one place, UserSearch.tml).
	// We do this by contributing to Tapestry's BindingSource service.
	// - Based on http://wiki.apache.org/tapestry/Tapestry5HowToAddBindingPrefix

	public static void contributeBindingSource(MappedConfiguration<String, BindingFactory> configuration,
			BindingSource bindingSource) {
		configuration.add("list", new ListBindingFactory(bindingSource));
	}

	// Tell Tapestry where to slot our ProtectedPageGateKeeper into its chain of dispatchers
	// We do this by contributing to Tapestry's Master Dispatcher.
	// - ProtectedPageGateKeeper prevents access to pages marked as @ProtectedPage unless logged in.
	// - Dispatchers are described in http://tapestry.apache.org/tapestry5/guide/request.html .
	// - Based on http://wiki.apache.org/tapestry/Tapestry5HowToControlAccess

	public void contributeMasterDispatcher(OrderedConfiguration<Dispatcher> configuration,
			@InjectService("ProtectedPageGateKeeper")
			Dispatcher protectedPageGateKeeper) {
		configuration.add("ProtectedPageGateKeeper", protectedPageGateKeeper, "before:PageRender");
	}
}
