package jumpstart.web.base.examples.wizard;

import jumpstart.web.commons.Conversation;
import jumpstart.web.commons.Conversations;
import jumpstart.web.pages.Index;
import jumpstart.web.pages.examples.wizard.WizardUsingPages;
import jumpstart.web.pages.examples.wizard.WizardUsingPagesApplicant;
import jumpstart.web.pages.examples.wizard.WizardUsingPagesBadFlow;
import jumpstart.web.pages.examples.wizard.WizardUsingPagesStart;
import jumpstart.web.pages.examples.wizard.WizardUsingPagesSubmit;
import jumpstart.web.pages.examples.wizard.WizardUsingPagesSuccess;
import jumpstart.web.state.examples.wizard.CreditRequest;

import org.apache.tapestry5.annotations.ApplicationState;
import org.apache.tapestry5.annotations.InjectPage;

public class WizardUsingPagesBase {

	public enum Step {
		START, APPLICANT, SUBMIT, SUCCESS, BAD_FLOW
	}

	private Step _step = null;

	private String _conversationId = null;

	@ApplicationState
	private Conversations _conversations;

	private CreditRequest _creditRequest;

	@InjectPage
	private WizardUsingPages _initPage;

	@InjectPage
	private WizardUsingPagesStart _startPage;

	@InjectPage
	private WizardUsingPagesApplicant _applicantPage;

	@InjectPage
	private WizardUsingPagesSubmit _submitPage;

	@InjectPage
	private WizardUsingPagesSuccess _successPage;

	@InjectPage
	private WizardUsingPagesBadFlow _badFlowPage;

	@InjectPage
	private Index _index;

	Object onSuccess() throws Exception {
		switch (_step) {
		case START:
			_applicantPage.set(_conversationId);
			return _applicantPage;
		case APPLICANT:
			_submitPage.set(_conversationId);
			return _submitPage;
		case SUBMIT:
			CreditRequest cr = getCreditRequestFromConversation(_conversationId);
			// In the real world you would pass the persisted credit request's id instead of these "approved" fields
			_successPage.set(cr.getAmount(), cr.getApplicantName());
			endConversation(_conversationId);
			return _successPage;
		default:
			throw new IllegalStateException("Should not get here. _step = " + _step);
		}
	}

	Object onPrev() throws Exception {
		switch (_step) {
		case APPLICANT:
			_startPage.set(_conversationId);
			return _startPage;
		case SUBMIT:
			_applicantPage.set(_conversationId);
			return _applicantPage;
		default:
			throw new IllegalStateException("Should not get here. _step = " + _step);
		}
	}

	Object onRestart() {
		return _initPage;
	}

	Object onQuit() {
		endConversation(_conversationId);
		return _index;
	}

	protected Object prepareNewConversationAndRedirectionToStart() throws Exception {
		String conversationId = startConversation(new CreditRequest());
		_startPage.set(conversationId);
		return _startPage;
	}

	protected Object activate(Step currentStep, String conversationId) throws Exception {
		_step = currentStep;
		_conversationId = conversationId;

		// If the requested conversation does not contains a credit request
		// then it means the Back/Reload/Refresh button has been used to reach an old conversation,
		// so redirect to the bad-flow-step

		_creditRequest = getCreditRequestFromConversation(conversationId);

		if (_creditRequest == null) {
			return _badFlowPage;
		}

		return null;
	}

	private String startConversation(Object target) {
		String conversationId = Long.toString(System.currentTimeMillis());
		_conversations.add(new Conversation(conversationId, target));
		return conversationId;
	}

	private void endConversation(String conversationId) {
		_conversations.remove(conversationId);

		// If conversations ASO is now empty then remove it from the session

		if (_conversations.isEmpty()) {
			_conversations = null;
		}
	}

	private CreditRequest getCreditRequestFromConversation(String conversationId) {
		Conversation conversation = _conversations.get(conversationId);
		if (conversation != null && conversation.getTarget() instanceof CreditRequest) {
			return (CreditRequest) conversation.getTarget();
		}
		return null;
	}

	protected Step getStep() {
		return _step;
	}

	protected void setStep(Step step) {
		_step = step;
	}

	protected String getConversationId() {
		return _conversationId;
	}

	protected void setConversationId(String conversationId) {
		_conversationId = conversationId;
	}

	public CreditRequest getCreditRequest() {
		return _creditRequest;
	}

	public void setCreditRequest(CreditRequest creditRequest) {
		_creditRequest = creditRequest;
	}

}
