package jumpstart.web.pages.examples.select;

import java.util.List;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.web.commons.GenericSelectModel;
import jumpstart.web.services.IBusinessServicesLocator;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ioc.services.PropertyAccess;

public class EasyObjectSelect {

	@Property
	@Persist
	private GenericSelectModel<Person> _persons;
	
	@Property
	private Person _person;
	
	@Inject
	private IBusinessServicesLocator _businessServicesLocator;
	
	@Inject
	private PropertyAccess _propertyAccess;

	Long onPassivate() {
		return _person == null ? null : _person.getId();
	}

	void onActivate(EventContext context) {
		if (_persons == null) {
			refreshPersons();
		}
		
		if (context.getCount() > 0) {
			Long personId = context.get(Long.class, 0);
			_person = findPersonInModel(personId);
		}
	}

	void refreshPersons() {
		// Get all persons - ask business service to find them (from the database)
		List<Person> persons = getPersonService().findPersons();
		_persons = new GenericSelectModel<Person>(persons, Person.class, "firstName", "id", _propertyAccess);
	}

	Person findPersonInModel(Long personId) {
		for (Person person : _persons.getList()) {
			if (person.getId().equals(personId)) {
				return person;
			}
		}
		return null; 
	}
	
	private IPersonServiceLocal getPersonService() {
		// Use our business services locator to get the EJB3 session bean called "PersonServiceLocal".
		return _businessServicesLocator.getPersonServiceLocal();
	}
	
}