package jumpstart.web.pages.examples.tables;

import java.text.DateFormat;
import java.text.Format;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.web.commons.ExceptionUtil;
import jumpstart.web.pages.Index;
import jumpstart.web.services.IBusinessServicesLocator;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.PrimaryKeyEncoder;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;

public class AjaxFormLoopTailored1 {

	// We've used "session" persistence but it is risky. Better techniques include wrapping this field in a
	// Conversation (see the Wizard examples) or persisting it in the database.
	@Property
	@Persist
	private List<PersonHolder> _personHolders;

	@SuppressWarnings("unused")
	@Property
	private PersonHolder _personHolder;

	private List<Person> _persons;

	@Component(id = "personsedit")
	private Form _form;

	@Inject
	private IBusinessServicesLocator _businessServicesLocator;

	@InjectPage
	private AjaxFormLoopTailored2 _page2;

	@Inject
	private Locale _currentLocale;

	@Inject
	private ComponentResources _resources;

	void onPrepare() {
		if (!_form.getHasErrors()) {
			if (_personHolders == null) {
				// Get all persons - ask business service to find them (from the database)
				_persons = getPersonService().findPersons();

				_personHolders = new ArrayList<PersonHolder>();
				for (Person person : _persons) {
					_personHolders.add(new PersonHolder(person, false, person.getId()));
				}
			}
		}
	}

	PersonHolder onAddRow() {
		// Create a skeleton Person and add it to the displayed list with a unique key
		Person newPerson = new Person();
		PersonHolder newPersonHolder = new PersonHolder(newPerson, true, 0 - System.nanoTime());
		_personHolders.add(newPersonHolder);

		return newPersonHolder;
	}

	void onRemoveRow(PersonHolder personPlus) {
		_personHolders.remove(personPlus);
	}

	void onValidateForm() {
		List<Person> personsToCreate = new ArrayList<Person>();

		for (PersonHolder holder : _personHolders) {
			if (holder.isNew()) {
				personsToCreate.add(holder.getPerson());
			}
		}

		System.out.println(">>> personsToCreate = " + personsToCreate);

		try {
			// In a real application you would persist them to the database instead of printing them
			// getPersonService().createPersons(personsToCreate);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			_form.recordError(ExceptionUtil.getRootCause(e));
		}
	}

	Object onSuccess() {
		List<Person> persons = new ArrayList<Person>();
		for (PersonHolder holder : _personHolders) {
			persons.add(holder.getPerson());
		}
		_page2.set(persons);
		_resources.discardPersistentFieldChanges();
		return _page2;
	}

	void onRefresh() {
		_resources.discardPersistentFieldChanges();
		onPrepare();
	}

	Object onActionFromGoHome() {
		_resources.discardPersistentFieldChanges();
		return Index.class;
	}

	@SuppressWarnings("unchecked")
	public PrimaryKeyEncoder getEncoder() {
		return new PrimaryKeyEncoder<Long, PersonHolder>() {

			public Long toKey(PersonHolder value) {
				return value.getKey();
			}

			public void prepareForKeys(List<Long> keys) {
			}

			public PersonHolder toValue(Long key) {
				for (PersonHolder holder : _personHolders) {
					if (holder.getKey().equals(key)) {
						return holder;
					}
				}
				throw new IllegalArgumentException("Received key \"" + key
						+ "\" which has no counterpart in this collection: " + _personHolders);
			}
		};
	}

	public class PersonHolder {
		private Person _person;
		private Long _key;
		private boolean _new;

		PersonHolder(Person person, boolean newPerson, Long key) {
			_person = person;
			_new = newPerson;
			_key = key;
		}

		public Person getPerson() {
			return _person;
		}

		public Long getKey() {
			return _key;
		}

		public boolean isNew() {
			return _new;
		}
	}

	private IPersonServiceLocal getPersonService() {
		// Use our business services locator to get the EJB3 session bean called "PersonServiceLocal".
		return _businessServicesLocator.getPersonServiceLocal();
	}

	public Format getDateFormat() {
		return DateFormat.getDateInstance(DateFormat.SHORT, _currentLocale);
	}

}
