package jumpstart.web.pages.examples.tables;

import java.util.List;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.web.commons.ExceptionUtil;
import jumpstart.web.services.IBusinessServicesLocator;

import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.util.DefaultPrimaryKeyEncoder;

public class EditableGridUsingEncoder1 {

	@Property
	private List<Person> _persons;

	@SuppressWarnings("unused")
	@Property
	private Person _person;

	@Property
	private DefaultPrimaryKeyEncoder<Long, Person> _encoder;

	@Component(id = "personsedit")
	private Form _form;

	@Inject
	private IBusinessServicesLocator _businessServicesLocator;

	@InjectPage
	private EditableGridUsingEncoder2 _page2;

	// Form triggers the PREPARE event during form render and form submission.
	void onPrepare() {
		// Get all persons - ask business service to find them (from the database)
		_persons = getPersonService().findPersons();

		// Load all the persons and their primary keys into the DefaultPrimaryKeyEncoder.
		_encoder = new DefaultPrimaryKeyEncoder<Long, Person>();
		for (Person person : _persons) {
			_encoder.add(person.getId(), person);
		}
	}

	void onValidateForm() {
		// Loop will already have updated every person in the encoder with a matching primary key.
		_persons = _encoder.getValues();

		try {
			System.out.println(">>> persons = " + _persons);
			// In a real application you would persist them to the database instead of printing them
			// getPersonService().changePersons(_persons);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			_form.recordError(ExceptionUtil.getRootCause(e));
		}
	}

	Object onSuccess() {
		_page2.set(_persons);
		return _page2;
	}

	private IPersonServiceLocal getPersonService() {
		// Use our business services locator to get the EJB3 session bean called "PersonServiceLocal".
		return _businessServicesLocator.getPersonServiceLocal();
	}
}
