package jumpstart.web.pages.examples.tables;

import java.util.ArrayList;
import java.util.List;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.web.pages.Index;
import jumpstart.web.services.IBusinessServicesLocator;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.ioc.annotations.Inject;

public class GridWithDeleteColumn1 {

	@Property
	// Oddly, in this example we have to persist the list, else on submit we get "Failure reading parameter 'source'" of the 
	// column names in the Grid. But persisting a field in the session is not ideal because the field will be overwritten if 
	// user opens new window with same session and visits the same page. 
	@Persist
	private List<Person> _persons;

	@Property
	private Person _person;

	@Inject
	private IBusinessServicesLocator _businessServicesLocator;

	private boolean _delete;

	private List<Person> _personsToDelete;

	@InjectPage
	private GridWithDeleteColumn2 _page2;

	@Inject
	private ComponentResources _resources;

	void setupRender() {
		// Get all persons - ask business service to find them (from the database)
		_persons = getPersonService().findPersons();
	}

	public void onPrepareForSubmit() {
		if (_persons == null) {
			_persons = new ArrayList<Person>();
		}
	}

	Object onSuccess() {
		_page2.set(getPersonsToDelete());
		_resources.discardPersistentFieldChanges();
		return _page2;
	}

	Object onActionFromGoHome() {
		_resources.discardPersistentFieldChanges();
		return Index.class;
	}

	private IPersonServiceLocal getPersonService() {
		// Use our business services locator to get the EJB3 session bean called "PersonServiceLocal".
		return _businessServicesLocator.getPersonServiceLocal();
	}

	public List<Person> getPersonsToDelete() {
		if (_personsToDelete == null) {
			_personsToDelete = new ArrayList<Person>();
		}
		return _personsToDelete;
	}

	// The Grid component will automatically call this for every row as it is rendered.
	public boolean isDelete() {
		return _delete;
	}

	// The Grid component will automatically call this for every row on submit.
	public void setDelete(boolean delete) {
		if (delete) {
			getPersonsToDelete().add(_person);
		}
	}
}
