package jumpstart.web.pages.examples.wizard;

import jumpstart.web.commons.Conversation;
import jumpstart.web.commons.Conversations;
import jumpstart.web.pages.Index;
import jumpstart.web.state.examples.wizard.CreditRequest;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.annotations.ApplicationState;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;

public class WizardUsingComponents {

	public enum Step {
		START, APPLICANT, SUBMIT, SUCCESS, BAD_FLOW
	}

	private Step _step = null;
	private String _conversationId = null;
	
	@ApplicationState
	private Conversations _conversations;

	@Property
	private CreditRequest _creditRequest;

	@Property
	private int _approvedAmount;

	@Property
	private String _approvedApplicantName;

	@InjectPage
	private Index _index;
	
	Object[] onPassivate() {

		if (_step == null) {
			return null;
		}
		else {
			switch (_step) {
			case START:
			case APPLICANT:
			case SUBMIT:
				return new Object[] { _step, _conversationId };
			case SUCCESS:
				// In the real world we would passivate the credit request's id instead of these fields
				return new Object[] { _step, _approvedAmount, _approvedApplicantName };
			case BAD_FLOW:
				return new Object[] { _step };
			default:
				throw new IllegalStateException("Should not get here");
			}
		}
	}

	Object onActivate(EventContext context) throws Exception {

		// If no context then start a new conversation with a new credit request and redirect to first step

		if (context.getCount() == 0) {
			_conversationId = startConversation(new CreditRequest());
			_step = Step.START;
			return this;
		}

		// Otherwise, the context will contain a step id and the rest of its contents will depend on the step id

		else {
			_step = context.get(Step.class, 0);

			switch (_step) {
			case START:
			case APPLICANT:
			case SUBMIT:
				assertContextParameterCount(context, 2);
				_conversationId = context.get(String.class, 1);

				// If the requested conversation does not contains a credit request
				// then it means the Back/Reload/Refresh button has been used to reach an old conversation,
				// so redirect to the bad-flow-step

				_creditRequest = getCreditRequestFromConversation(_conversationId);
				if (_creditRequest == null) {
					_step = Step.BAD_FLOW;
					return this;
				}
				break;
			case SUCCESS:
				assertContextParameterCount(context, 3);
				_approvedAmount = context.get(Integer.class, 1);
				_approvedApplicantName = context.get(String.class, 2);
				break;
			case BAD_FLOW:
				// Nothing to do
				break;
			default:
				throw new IllegalStateException("Should not get here");
			}
		}

		return null;
	}

	private void assertContextParameterCount(EventContext context, int count) throws Exception {
		if (context.getCount() != count) {
			throw new Exception("Found context.count = " + context.getCount() + ".");
		}
	}

	void onSuccess() {
		switch (_step) {
		case START:
			_step = Step.APPLICANT;
			break;
		case APPLICANT:
			_step = Step.SUBMIT;
			break;
		case SUBMIT:
			_step = Step.SUCCESS;

			// In the real world we would now save the credit request's id instead of these fields

			_approvedAmount = _creditRequest.getAmount();
			_approvedApplicantName = _creditRequest.getApplicantName();

			endConversation(_conversationId);
			break;
		default:
			throw new IllegalStateException("Should not get here. _step = " + _step);
		}
	}

	void onPrev() {
		switch (_step) {
		case APPLICANT:
			_step = Step.START;
			break;
		case SUBMIT:
			_step = Step.APPLICANT;
			break;
		default:
			throw new IllegalStateException("Should not get here. _step = " + _step);
		}
	}

	void onRestart() {
		_step = null;
	}

	Object onQuit() {
		endConversation(_conversationId);
		return _index;
	}

	private String startConversation(Object target) {
		String conversationId = Long.toString(System.currentTimeMillis());
		_conversations.add(new Conversation(conversationId, target));
		return conversationId;
	}

	private void endConversation(String conversationId) {
		_conversations.remove(conversationId);

		// If conversations ASO is now empty then remove it from the session
		
		if (_conversations.isEmpty()) {
			_conversations = null;
		}
	}

	private CreditRequest getCreditRequestFromConversation(String conversationId) {
		Conversation conversation = _conversations.get(conversationId);
		if (conversation != null && conversation.getTarget() instanceof CreditRequest) {
			return (CreditRequest) conversation.getTarget();
		}
		return null;
	}

	public boolean isInStart() {
		return _step == Step.START;
	}

	public boolean isInApplicant() {
		return _step == Step.APPLICANT;
	}

	public boolean isInSubmit() {
		return _step == Step.SUBMIT;
	}

	public boolean isInSuccess() {
		return _step == Step.SUCCESS;
	}

	public boolean isInBadFlow() {
		return _step == Step.BAD_FLOW;
	}

}