package jumpstart.web.pages.theapp.security;

import java.util.List;

import jumpstart.business.commons.exception.BusinessException;
import jumpstart.business.commons.exception.DoesNotExistException;
import jumpstart.business.domain.security.User;
import jumpstart.business.domain.security.UserRole;
import jumpstart.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.web.annotation.ProtectedPage;
import jumpstart.web.base.theapp.SimpleBasePage;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.Link;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.ioc.annotations.Inject;

@ProtectedPage
public class MyAccountView extends SimpleBasePage {

	// The User
	
	@Property
	private User _user;

	// The User's Roles
	
	@SuppressWarnings("unused")
	@Property
	private List<UserRole> _userRoles;

	@SuppressWarnings("unused")
	@Property
	private UserRole _userRole;

	// Other pages
	
	@InjectPage
	private UserRoleView _viewPage;

	// Generally useful bits and pieces.
	
	@Inject
	private ComponentResources _resources;

	void onActivate() throws DoesNotExistException {
		Long userId = getVisit().getMyUserId();
		_user = getSecurityFinderService().findUser(userId);
	}

	void setupRender() throws BusinessException {
		_userRoles = getSecurityFinderService().findUserRolesShallowishByUser(_user.getId());
	}

	Object onEdit() {
		return MyAccountEdit.class;
	}

	Object onChangePassword() {
		return MyPasswordChange.class;
	}

	Object onActionFromViewUserRole(Long id) {
		_viewPage.set(id, createLinkToThisPage());
		return _viewPage;
	}

	private Link createLinkToThisPage() {
		Link thisPageLink = _resources.createPageLink(_resources.getPageName(), false);
		return thisPageLink;
	}

	private ISecurityFinderSvcLocal getSecurityFinderService() {
		return getBusinessServicesLocator().getSecurityFinderSvcLocal();
	}
}
