package jumpstart.web.pages.theapp.security;

import java.util.List;

import jumpstart.business.commons.exception.DoesNotExistException;
import jumpstart.business.domain.security.Role;
import jumpstart.business.domain.security.UserRole;
import jumpstart.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.web.annotation.ProtectedPage;
import jumpstart.web.base.theapp.SimpleBasePage;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.Link;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;

@ProtectedPage
public class RoleEdit extends SimpleBasePage {

	// The Role
	
	@Property
	private Long _roleId;

	@Property
	private Role _role;

	// The Role's Users
	
	@SuppressWarnings("unused")
	@Property
	private List<UserRole> _userRoles;

	@SuppressWarnings("unused")
	@Property
	private UserRole _userRole;

	// Other pages
	
	@InjectPage
	private UserRoleAdd _addPage;

	@InjectPage
	private UserRoleView _viewPage;

	@InjectPage
	private UserRoleEdit _editPage;

	// Generally useful bits and pieces.
	
	@Component(id = "form")
	private Form _form;

	@Inject
	private ComponentResources _resources;

	public void set(Long roleId) {
		_roleId = roleId;
	}
	
	Long onPassivate() {
		return _roleId;
	}

	void onActivate(Long roleId) throws DoesNotExistException {
		_roleId = roleId;
		try {
			_role = getSecurityFinderService().findRole(_roleId);
		}
		catch (DoesNotExistException e) {
			// Handle null role in the template
		}
	}

	void setupRender() {
		_userRoles = getSecurityFinderService().findUserRolesShallowishByRole(_roleId);
	}

	void onValidateForm() {
		try {
			getSecurityManagerService().changeRole(_role);
		}
		catch (Exception e) {
			_form.recordError(interpretBusinessServicesExceptionForChange(e));
		}
	}

	Object onSuccess() {
		return RoleSearch.class;
	}

	void onRefresh() {
	}

	Object onCancel() {
		return RoleSearch.class;
	}

	Object onAddUserRole() {
		_addPage.set(null, _roleId, createLinkToThisPage());
		return _addPage;
	}

	Object onActionFromViewUserRole(Long id) {
		_viewPage.set(id, createLinkToThisPage());
		return _viewPage;
	}

	Object onActionFromEditUserRole(Long id) {
		_editPage.set(id, createLinkToThisPage());
		return _editPage;
	}

	void onActionFromRemoveUserRole(Long id, Integer version) {

		if (!_form.getHasErrors()) {

			// Delete the user from the database unless they've been modified elsewhere

			try {
				UserRole userRole = getSecurityFinderService().findUserRole(id);
				if (!userRole.getVersion().equals(version)) {
					_form
							.recordError("Cannot remove user role because has been updated or deleted since last displayed.  Please refresh and try again.");
					return;
				}
				getSecurityManagerService().removeUserRole(userRole);
			}
			catch (Exception e) {
				_form.recordError(interpretBusinessServicesExceptionForRemove(e));
				return;
			}
		}
	}

	private Link createLinkToThisPage() {
		Link thisPageLink = _resources.createPageLink(_resources.getPageName(), false, onPassivate());
		return thisPageLink;
	}

	private ISecurityFinderSvcLocal getSecurityFinderService() {
		return getBusinessServicesLocator().getSecurityFinderSvcLocal();
	}

	private ISecurityManagerSvcLocal getSecurityManagerService() {
		return getBusinessServicesLocator().getSecurityManagerSvcLocal();
	}
}
