package jumpstart.web.pages.theapp.security;

import java.util.List;

import jumpstart.business.commons.exception.DoesNotExistException;
import jumpstart.business.domain.security.User;
import jumpstart.business.domain.security.UserRole;
import jumpstart.business.domain.security.User.PageStyle;
import jumpstart.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.web.annotation.ProtectedPage;
import jumpstart.web.base.theapp.SimpleBasePage;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.Link;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.util.EnumValueEncoder;

@SuppressWarnings("unchecked")
@ProtectedPage
public class UserEdit extends SimpleBasePage {

	// The User

	@Property
	private Long _userId;

	@Property
	private User _user;

	// The User's Roles

	@SuppressWarnings("unused")
	@Property
	private List<UserRole> _userRoles;

	@SuppressWarnings("unused")
	@Property
	private UserRole _userRole;

	// Other pages

	@InjectPage
	private UserSearch _userSearch;
	
	@InjectPage
	private UserPasswordChange _userPasswordChange;

	@InjectPage
	private UserRoleAdd _addPage;

	@InjectPage
	private UserRoleView _viewPage;

	@InjectPage
	private UserRoleEdit _editPage;
	
	// Generally useful bits and pieces.

	@Component(id = "form")
	private Form _form;

	@Inject
	private ComponentResources _resources;
	
	@SuppressWarnings("unused")
	@Property
	private EnumValueEncoder _pageStyleEncoder = new EnumValueEncoder(User.PageStyle.class);

	public void set(Long id) {
		_userId = id;
	}

	Long onPassivate() {
		return _userId;
	}

	void onActivate(Long id) {
		_userId = id;
		try {
			_user = getSecurityFinderService().findUser(_userId);
		}
		catch (DoesNotExistException e) {
			// Handle null user in the template
		}
	}

	void setupRender() {
		_userRoles = getSecurityFinderService().findUserRolesShallowishByUser(_userId);
	}

	void onValidateForm() {
		try {
			getSecurityManagerService().changeUser(_user);
		}
		catch (Exception e) {
			_form.recordError(interpretBusinessServicesExceptionForChange(e));
		}
	}

	Object onSuccess() {
		getVisit().noteChanges(_user);
		return _userSearch.createLinkWithLastSearch();
	}
	
	Object onChangePassword() {
		_userPasswordChange.set(_userId) ;
		return _userPasswordChange;
	}

	void onRefresh() {
	}

	Object onCancel() {
		return _userSearch.createLinkWithLastSearch();
	}

	Object onAddUserRole() {
		_addPage.set(_userId, null, createLinkToThisPage());
		return _addPage;
	}

	Object onActionFromViewUserRole(Long id) {
		_viewPage.set(id, createLinkToThisPage());
		return _viewPage;
	}

	Object onActionFromEditUserRole(Long id) {
		_editPage.set(id, createLinkToThisPage());
		return _editPage;
	}

	void onActionFromRemoveUserRole(Long id, Integer version) {

		if (!_form.getHasErrors()) {

			// Delete the user from the database unless they've been modified elsewhere

			try {
				UserRole userRole = getSecurityFinderService().findUserRole(id);
				if (!userRole.getVersion().equals(version)) {
					_form
							.recordError("Cannot remove user role because has been updated or deleted since last displayed.  Please refresh and try again.");
					return;
				}
				getSecurityManagerService().removeUserRole(userRole);
			}
			catch (Exception e) {
				_form.recordError(interpretBusinessServicesExceptionForRemove(e));
				return;
			}
		}
	}

	private Link createLinkToThisPage() {
		Link thisPageLink = _resources.createPageLink(_resources.getPageName(), false, onPassivate());
		return thisPageLink;
	}

	private ISecurityFinderSvcLocal getSecurityFinderService() {
		return getBusinessServicesLocator().getSecurityFinderSvcLocal();
	}

	private ISecurityManagerSvcLocal getSecurityManagerService() {
		return getBusinessServicesLocator().getSecurityManagerSvcLocal();
	}

	public String[] getSalutations() {
		return User.SALUTATIONS;
	}

	public PageStyle getBoxy() {
		return User.PageStyle.BOXY;
	}

	public PageStyle getWide() {
		return User.PageStyle.WIDE;
	}

	public String[] getDatePatterns() {
		return User.DATE_PATTERNS;
	}
}
