package jumpstart.web.services;

import javax.naming.NamingException;

import jumpstart.business.domain.examples.iface.IDateStuffServiceLocal;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.util.EJBProviderEnum;
import jumpstart.util.EJBProviderUtil;
import jumpstart.util.JNDIServicesLocator;

import org.slf4j.Logger;

/**
 * BusinessServicesLocator is used to centralize all JNDI lookups. It is a global singleton registered in AppModule. It
 * minimises the overhead of JNDI lookups by caching the objects it looks up. If this class becomes a bottleneck, then
 * you may need to decentralise it.
 */
public class BusinessServicesLocator extends JNDIServicesLocator implements IBusinessServicesLocator {

	protected String _personServiceName, _dateStuffServiceName, _securityFinderSvcName, _securityManagerSvcName;

	public BusinessServicesLocator(Logger logger) throws NamingException {
		super(logger);
		loadUpServiceNames(logger);
	}

	public IPersonServiceLocal getPersonServiceLocal() {
		return (IPersonServiceLocal) getJndiService(_personServiceName);
	}

	public IDateStuffServiceLocal getDateStuffServiceLocal() {
		return (IDateStuffServiceLocal) getJndiService(_dateStuffServiceName);
	}

	public ISecurityFinderSvcLocal getSecurityFinderSvcLocal() {
		return (ISecurityFinderSvcLocal) getJndiService(_securityFinderSvcName);
	}

	public ISecurityManagerSvcLocal getSecurityManagerSvcLocal() {
		return (ISecurityManagerSvcLocal) getJndiService(_securityManagerSvcName);
	}

	private void loadUpServiceNames(Logger logger) {
		EJBProviderEnum ejbProvider = EJBProviderUtil.detectEJBProvider(logger);

		if (ejbProvider == EJBProviderEnum.OPENEJB_LOCAL || ejbProvider == EJBProviderEnum.TOMCAT_OPENEJB_LOCAL) {
			_personServiceName = "PersonServiceLocal";
			_dateStuffServiceName = "DateStuffServiceLocal";
			_securityFinderSvcName = "SecurityFinderSvcLocal";
			_securityManagerSvcName = "SecurityManagerSvcLocal";
		}
		else if (ejbProvider == EJBProviderEnum.JBOSS_LOCAL) {
			_personServiceName = "jumpstart/PersonService/local";
			_dateStuffServiceName = "jumpstart/DateStuffService/local";
			_securityFinderSvcName = "jumpstart/SecurityFinderSvc/local";
			_securityManagerSvcName = "jumpstart/SecurityManagerSvc/local";
		}
		else if (ejbProvider == EJBProviderEnum.GLASSFISH_LOCAL) {
			// Local interfaces in Glassfish are a bit touchy: the JNDI name used here must match the ejb-ref-name in
			// web.xml. See https://glassfish.dev.java.net/javaee5/ejb/EJB_FAQ.html
			_personServiceName = "java:comp/env/PersonService";
			_dateStuffServiceName = "java:comp/env/DateStuffService";
			_securityFinderSvcName = "java:comp/env/SecurityFinderSvc";
			_securityManagerSvcName = "java:comp/env/SecurityManagerSvc";
		}
		else {
			throw new IllegalStateException("Don't know how to use ejbProvider = " + ejbProvider);
		}
	}
}
